#ifndef HARKIO_HARK_HGTF_FMT_HPP_
#define HARKIO_HARK_HGTF_FMT_HPP_

#include <string>
#include <vector>
#include <harkio2/Version.hpp>
#include <harkio2/Vector3.hpp>
#include <harkio2/defineitem.hpp>
#include <harkio2/hark/DataSet.hpp>
#include <harkio2/InvalidHarkFile.hpp>
#include <harkio2/hark/HarkFileWriter.hpp>

/** @file Fmt.hpp
 * @brief Fmt class and subclasses definitions.
 */

namespace harkio2
{
namespace hark
{
namespace hgtf
{

/** @brief Abstract format data set.
 * This class represents format information such as
 * the number of microphones and the FFT length.
 */
class Fmt : public harkio2::hark::DataSet
{
public:
    /** @brief Returns the type identifier.
     * @return The type identifier
     */
    virtual std::string GetId() const { return "hgtf::Fmt"; }

    /** @brief Returns the version.
     * Returns the version which this instance actually represents.
     * @return The version number.
     */
    virtual harkio2::Version GetVersion() const = 0;
};

/** @brief Format data set version 1.0.
 */
class Fmt1_0 : public Fmt
{
public:
    /** @brief Returns the version.
     * @return The version which this class represents.
     */
    virtual harkio2::Version GetVersion() const { return harkio2::Version(1, 0); }

    DEFINE_ITEM(formatId, unsigned int, FormatId, unsigned int)
    DEFINE_ITEM(nMic, unsigned int, NMic, unsigned int)
    DEFINE_ITEM(nFFT, unsigned int, NFFT, unsigned int)
    DEFINE_ITEM(nImpulseResponse, unsigned int, NImpulseResponse, unsigned int)
    DEFINE_ITEM(fs, unsigned int, FS, unsigned int)
    DEFINE_ITEM(micPositions, std::vector<harkio2::Vector3<float> >, MicPositions, const std::vector<harkio2::Vector3<float> >&)

    /// @brief Implementation of Writable interface.
    void Write(HarkFileWriter& writer) throw (harkio2::InvalidHarkFile)
    {
        writer.Write(*this);
    }

};

}
}
}

#endif // HARKIO_HARK_HGTF_FMT_HPP_

