#ifndef HARKIO_HARK_HGTF_SPEC_HPP_
#define HARKIO_HARK_HGTF_SPEC_HPP_

#include <string>
#include <glibmm/ustring.h>
#include <harkio2/Version.hpp>
#include <harkio2/defineitem.hpp>
#include <harkio2/hark/DataSet.hpp>
#include <harkio2/InvalidHarkFile.hpp>
#include <harkio2/hark/HarkFileWriter.hpp>

/** @file Spec.hpp
 * @brief Spec class and subclasses definitions.
 */

namespace harkio2
{
namespace hark
{
namespace hgtf
{

/** @brief Abstract specification data set.
 * This class represents specification such as
 * creation date and location.
 */
class Spec : public harkio2::hark::DataSet
{
public:
    /** @brief Returns the type identifier.
     * @return The type identifier
     */
    virtual std::string GetId() const { return "hgtf::Spec"; }

    /** @brief Returns the version.
     * Returns the version which this instance actually represents.
     * @return The version number.
     */
    virtual harkio2::Version GetVersion() const = 0;
};

/** @brief Format data set version 1.0.
 */
class Spec1_0 : public Spec
{
public:
    /** @brief Returns the version.
     * @return The version which this class represents.
     */
    virtual harkio2::Version GetVersion() const { return harkio2::Version(1, 0); }

    DEFINE_ITEM(date, Glib::ustring, Date, const Glib::ustring&)
    DEFINE_ITEM(location, Glib::ustring, Location, const Glib::ustring&)
    DEFINE_ITEM(robot, Glib::ustring, Robot, const Glib::ustring&)
    DEFINE_ITEM(micArrayType, Glib::ustring, MicArrayType, const Glib::ustring&)
    DEFINE_ITEM(creator, Glib::ustring, Creator, const Glib::ustring&)
    DEFINE_ITEM(memo, Glib::ustring, Memo, const Glib::ustring&)

    /// @brief Implementation of Writable interface.
    void Write(HarkFileWriter& writer) throw (harkio2::InvalidHarkFile)
    {
        writer.Write(*this);
    }

};

}
}
}

#endif // HARKIO_HARK_HGTF_SPEC_HPP_

