#ifndef HARKIO_HARKCONFIG_NOISELOCATIONFILE_HPP_
#define HARKIO_HARKCONFIG_NOISELOCATIONFILE_HPP_

#include <string>
#include <vector>
#include <glibmm/ustring.h>
#include <harkio2/Version.hpp>
#include <harkio2/defineitem.hpp>
#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/NoiseSource.hpp>
#include <harkio2/harkconfig/HarkConfigFileWriter.hpp>

/** @file HarkConfigFile.hpp
 * @brief HarkConfigFile class and subclasses definitions.
 */

namespace harkio2
{
namespace harkconfig
{

/** @brief Abstract Noise Location file.
 * This class represents a file which contains a array of noise locations.
 * The file can be any version.
 */
class NoiseLocationFile : public HarkConfigFile
{
public:
    /** @brief Returns the file type identifier.
     * @return The file type identifier which this class represents.
     */
    virtual std::string GetId() const { return "NoiseLocationFile"; }
};

/** @brief Noise Location file version 2.0.
 */
class NoiseLocationFile2_0 : public NoiseLocationFile
{
public:
    /** @brief Returns the file version.
     * @return The version which this class represents.
     */
    virtual harkio2::Version GetVersion() const { return Version(2, 0); }

    DEFINE_ITEM(comment, Glib::ustring, Comment, const Glib::ustring&)
    DEFINE_ITEM(noiseSources, std::vector<harkio2::harkconfig::NoiseSource1>, NoiseSources, const std::vector<harkio2::harkconfig::NoiseSource1>&)

    /// @brief Implementation of Writable interface.
    void Write(HarkConfigFileWriter& writer) const throw (harkio2::InvalidHarkFile)
    {
        writer.Write(*this);
    }

};

}
}

#endif // HARKIO_HARKCONFIG_NOISELOCATIONFILE_HPP_

