#include <harkio2/harkconfig/HarkConfigFileXMLReader.hpp>

#include <map>
#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>
#include <libxml++/libxml++.h>
#include <libxml++/parsers/textreader.h>

#include <harkio2/harkconfig/nodeparsers.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFile.hpp>
#include <harkio2/harkconfig/TSPListFile.hpp>
#include <harkio2/harkconfig/MicArrayLocationFile.hpp>
#include <harkio2/harkconfig/NoiseLocationFile.hpp>
#include <harkio2/xmlhelper/xmlhelper.hpp>

using namespace std;
using namespace boost;
using namespace xmlpp;
using namespace Glib;

namespace harkio2
{
namespace harkconfig
{

HarkConfigFileXMLReader::HarkConfigFileXMLReader(const Glib::ustring& uri) :
    uri(uri)
{}

boost::shared_ptr<HarkConfigFile> HarkConfigFileXMLReader::Read()
    throw (harkio2::AnalyzeError)
{
    try
    {
        TextReader reader(uri);

        try
        {
            harkio2::xmlhelper::MoveToNextElement(reader);

            if (harkio2::xmlhelper::IsElement(reader, "hark_xml"))
            {
                if (harkio2::xmlhelper::FindElement(reader, "hark_config"))
                {
                    typedef map<Glib::ustring, Glib::ustring> attr_t;
                    attr_t attrs;

                    while (reader.move_to_next_attribute())
                    {
                        attrs[reader.get_name()] = reader.get_value();
                    }

                    reader.move_to_element(); // move to the start element that contains the current attribute.

                    if (!harkio2::xmlhelper::Contains(attrs, "id"))
                    {
                        throw AnalyzeError("HarkConfigFileXMLReader::Read: 'hark_config' tag doesn't have 'id' attribute", harkio2::xmlhelper::GetLine(reader));
                    }

                    else if (attrs["id"] == "ImpulseResponseList")
                    {
                        if (attrs["major"] == "2" && attrs["minor"] == "0")
                        {
                            return ParseImpulseResponseListFile2_0(reader);
                        }
                        else
                        {
                            throw AnalyzeError("HarkConfigFileXMLReader::Read: invalid version for the specified 'id'", harkio2::xmlhelper::GetLine(reader));
                        }
                    }

                    else if (attrs["id"] == "TSPList")
                    {
                        if (attrs["major"] == "2" && attrs["minor"] == "0")
                        {
                            return ParseTSPListFile2_0(reader);
                        }
                        else
                        {
                            throw AnalyzeError("HarkConfigFileXMLReader::Read: invalid version for the specified 'id'", harkio2::xmlhelper::GetLine(reader));
                        }
                    }

                    else if (attrs["id"] == "MicArrayLocation")
                    {
                        if (attrs["major"] == "2" && attrs["minor"] == "0")
                        {
                            return ParseMicArrayLocationFile2_0(reader);
                        }
                        else
                        {
                            throw AnalyzeError("HarkConfigFileXMLReader::Read: invalid version for the specified 'id'", harkio2::xmlhelper::GetLine(reader));
                        }
                    }

                    else if (attrs["id"] == "NoiseLocation")
                    {
                        if (attrs["major"] == "2" && attrs["minor"] == "0")
                        {
                            return ParseNoiseLocationFile2_0(reader);
                        }
                        else
                        {
                            throw AnalyzeError("HarkConfigFileXMLReader::Read: invalid version for the specified 'id'", harkio2::xmlhelper::GetLine(reader));
                        }
                    }

                    else
                    {

                        throw AnalyzeError("HarkConfigFileXMLReader::Read: invalid 'id'", harkio2::xmlhelper::GetLine(reader));
                    }
                }
                else
                {

                    throw AnalyzeError("HarkConfigFileXMLReader::Read: can't find 'hark_config' tag", harkio2::xmlhelper::GetLine(reader));
                }
            }
            else
            {
                throw AnalyzeError("HarkConfigFileXMLReader::Read: This document is not valid Hark XML document", harkio2::xmlhelper::GetLine(reader));
            }

        }
        catch (const xmlpp::exception& e)
        {
            throw AnalyzeError((format("HarkConfigFileXMLReader::Read: %s") % e.what()).str(), harkio2::xmlhelper::GetLine(reader));
        }
        catch (const boost::bad_lexical_cast& e)
        {
            throw AnalyzeError((format("HarkConfigFileXMLReader::Read: %s") % e.what()).str(), harkio2::xmlhelper::GetLine(reader));
        }

    }
    catch (const xmlpp::exception& e)
    {
        throw AnalyzeError((format("HarkConfigFileXMLReader::Read: %s") % e.what()).str());
    }

    throw AnalyzeError("HarkConfigFileXMLReader::Read: can't read this file for unknown reasons");
}

}
}

