#define BOOST_TEST_MODULE testcase for ImpulseResponseListFile class
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <vector>
#include <boost/shared_ptr.hpp>
#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFile.hpp>
#include <harkio2/FileType.hpp>

using namespace harkio2;
using namespace harkio2::harkconfig;
using namespace boost;
using namespace std;

typedef boost::shared_ptr<HarkConfigFile> hcptr;
typedef boost::shared_ptr<ImpulseResponseListFile> irptr;
typedef boost::shared_ptr<ImpulseResponseListFile1_0> ir1_0ptr;
typedef boost::shared_ptr<ImpulseResponseListFile2_0> ir2_0ptr;

BOOST_AUTO_TEST_CASE(constructor_test)
{
    hcptr hc(new ImpulseResponseListFile1_0());

    BOOST_CHECK_EQUAL(hc->GetId() , "ImpulseResponseListFile");
    BOOST_CHECK(hc->GetVersion() == Version(1, 0));

    BOOST_CHECK(dynamic_pointer_cast<ImpulseResponseListFile>(hc) != 0);
    irptr ir = dynamic_pointer_cast<ImpulseResponseListFile>(hc);

    BOOST_CHECK(dynamic_pointer_cast<ImpulseResponseListFile1_0>(ir) != 0);

    hcptr hc2(new ImpulseResponseListFile2_0());

    BOOST_CHECK_EQUAL(hc2->GetId() , "ImpulseResponseListFile");
    BOOST_CHECK(hc2->GetVersion() == Version(2, 0));

    BOOST_CHECK(dynamic_pointer_cast<ImpulseResponseListFile>(hc2) != 0);
    irptr ir2 = dynamic_pointer_cast<ImpulseResponseListFile>(hc2);

    BOOST_CHECK(dynamic_pointer_cast<ImpulseResponseListFile2_0>(ir2) != 0);
}

BOOST_AUTO_TEST_CASE(accessor_exception_test)
{
    ir1_0ptr ir(new ImpulseResponseListFile1_0());

    BOOST_CHECK_THROW(ir->GetSources(), NotInitialized);

    vector<Source1> sources;
    ir->SetSources(sources);

    BOOST_CHECK_NO_THROW(ir->GetSources());

    ir2_0ptr ir2(new ImpulseResponseListFile2_0());

    BOOST_CHECK_THROW(ir2->GetSourceFileType(), NotInitialized);
    BOOST_CHECK_THROW(ir2->GetSources(), NotInitialized);

    vector<Source2> sources2;
    ir2->SetSourceFileType(FileType::WAVE);
    ir2->SetSources(sources2);

    BOOST_CHECK_NO_THROW(ir2->GetSourceFileType());
    BOOST_CHECK_NO_THROW(ir2->GetSources());

}

BOOST_AUTO_TEST_CASE(accessor_test)
{
    ir1_0ptr ir(new ImpulseResponseListFile1_0());

    vector<Source1> sources;
    ir->SetSources(sources);

    //BOOST_CHECK(ir->GetSources() == vector<Source1>());

    sources.push_back(Source1());
    sources.back().SetPos(Vector3<float>(100, 200, 300));

    //BOOST_CHECK(ir->GetSources() != sources);

    ir->SetSources(sources);

    //BOOST_CHECK(ir->GetSources() == sources);
    BOOST_CHECK(ir->GetSources().back().GetPos() == Vector3<float>(100, 200, 300));
    BOOST_CHECK(ir->GetSources().back().GetPos() != Vector3<float>(101, 200, 300));
}

